/*
** Module   : TemporaryAccount.java
** Abstract : a temporary account structure.
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131211 Created initial version.
** 002 MAG 20140128 Add timestamp field and check for expired.
*/

package com.goldencode.p2j.main;

import java.util.*;

/**
 * This structure holds the temporary account subject and the clear text password 
 * random generated. The structure is used as a pool item on temporary accounts pool 
 * implementation. This parameters are passed to the spawned process via environment variables.
 * Because the values are random generated and could contains special characters in order to 
 * avoid any side effects the values are converted in hex strings before stored as environment 
 * variables. On the spawned process the values are get from the process environment and decoded
 * from hex strings to their original values.
 * 
 * @author mag
 *
 */
public class TemporaryAccount
{
   /** Expired time in miliseconds */
   public static final long EXPIRED = 10000;

   /** Account subject id environment variable */
   public static final String P2J_SUBJECT = "P2J_SUBJECT";
   
   /** Account password environment variable */
   public static final String P2J_PASSWORD = "P2J_PASSWORD";
   
   /** Subject Id */
   private String subject;
   
   /** Password */
   private String password;
   
   /** Time stamp when the account was last enabled. */
   private long timestamp;

   /**
    * Convert a string into a hex format.
    * 
    * @param   text
    *          Input text.
    * 
    * @return  The converted string.
    */
   public static String toHex(String text)
   {
      StringBuilder buffer = new StringBuilder();
      
      for (byte b : text.getBytes())
      {
         buffer.append(String.format("%02X", b));
      }
      
      return buffer.toString();
   }
   
   /**
    * Convert a string from a hex format in ASCII
    * 
    * @param   text
    *          Input text. The length of the input must be multiple of 2.
    *          
    * @return  The converted string.
    */
   public static String fromHex(String text)
   {
      int length = text.length();
      
      if (length % 2 != 0)
      {
         throw new IllegalArgumentException("The input length must be multiple of 2");
      }
      
      StringBuilder buffer = new StringBuilder();
  
      for (int i = 0; i < length-1; i += 2 )
      {
          String hexPairs = text.substring(i, (i + 2));
          buffer.append((char)Integer.parseInt(hexPairs, 16));
      }
  
      return buffer.toString();   
   }

   /**
    * Constructor
    * 
    * @param   subject
    *          Subject id.
    * @param   password
    *          Password
    */
   public TemporaryAccount(String subject, String password)
   {
      this.password = password;      
      this.subject = subject;
   }
   
   /**
    * Create environment variables.
    *  
    * @return  A Map containing the random account credentials. 
    *          The key is the name of the environment variable.
    *          The values is the account value as hex format.
    */
   public Map<String, String> getEnvironmentMap()
   {
      Map<String, String> environment = new HashMap<>();
      environment.put(TemporaryAccount.P2J_SUBJECT, toHex(subject));
      environment.put(TemporaryAccount.P2J_PASSWORD, toHex(password));
      return environment;
   }
   
   /**
    * Get an item from pool
    * 
    * @return  fetch an item from the head of pool.
    */
   public static TemporaryAccount open()
   {
      return TemporaryAccountPool.getInstance().poll();
   }
   
   /**
    * Enqueued <code>this</code> item at the tail of the pool.
    *   
    */
   public void close()
   {
      TemporaryAccountPool.getInstance().tail(this);
   }

   /**
    * Get user account name.
    * 
    * @return  User account name.
    */
   public String getSubject()
   {
      return subject;
   }

   /**
    * Get account password.
    * 
    * @return  Account password.
    */
   public String getPassword()
   {
      return password;
   }

   /**
    * Set account password.
    * 
    * @param   password
    *          New password.
    */
   public void setPassword(String password)
   {
      this.password = password;
      timestamp = System.currentTimeMillis();
   }
   
   /**
    * Check if the account is enabled for an amount of time.
    * 
    * @return  <true> if the account was enabled from a specified amount of time
    *          <false> otherwise.
    */
   public boolean isExpired()
   {
      return (System.currentTimeMillis() - timestamp) > EXPIRED;
   }
}
